Package: autoharp
Title: Semi-Automatic Grading of R and Rmd Scripts
Version: 0.3.2
Authors@R: c(
    person("Vik", "Gopal", , "vik.gopal@nus.edu.sg", role = "aut"),
    person("Agrawal", "Naman", , "naman.a@nus.edu.sg", role = c("aut", "cre")),
    person("Yuting", "Huang", , "yhuang@nus.edu.sg", role = "ctb"),
    person("Samuel", "Seah", , "samuelseah@u.nus.edu", role = "aut"),
    person("Viknesh", "Jeya Kumar", , "viknesh@u.nus.edu", role = "aut"),
    person("Gabriel", "Ang", , "gabrielang@u.nus.edu", role = "aut"),
    person("Ruofan", "Liu", , "kelseyliu1998@gmail.com", role = "ctb"),
    person("National University of Singapore", role = "cph")
  )
Description: A customisable set of tools for assessing and grading 
    R or R-markdown scripts from students. It allows for checking correctness 
    of code output, runtime statistics and static code analysis. The latter 
    feature is made possible by representing R expressions using a tree
    structure.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3.9000
Imports: magrittr, dplyr, stringr, rlang, tibble, knitr, rmarkdown,
        shiny, lintr, methods, igraph, xfun
Collate: 'treeharp.R' 'th_getter-length.R' 'as.matrix.R' 'autoharp.R'
        'check_correctness.R' 'check_rmd.R' 'check_runtime.R'
        'count_lints.R' 'env_size.R' 'extracton_section_text.R'
        'forestharp.R' 'forestharp_helpers.R' 'generate_thumbnails.R'
        'join_treeharp.R' 'lang_2_tree.R' 'lang_2_tree_helpers.R'
        'log_summary.R' 'nlp_related.R' 'populate_soln_env.R'
        'pre_checks.R' 'render_one.R' 'reset_path.R' 'to_BFS.R'
        'tree_kernel.R' 'tree_routines.R' 'utils-pipe.R' 'utils.R'
        'write_html.R' 'zzz.R'
Suggests: testthat
SystemRequirements: pandoc (>= 1.14) - http://pandoc.org
URL: https://namanlab.github.io/autoharp/
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-11 08:03:48 UTC; namanagrawal
Author: Vik Gopal [aut],
  Agrawal Naman [aut, cre],
  Yuting Huang [ctb],
  Samuel Seah [aut],
  Viknesh Jeya Kumar [aut],
  Gabriel Ang [aut],
  Ruofan Liu [ctb],
  National University of Singapore [cph]
Maintainer: Agrawal Naman <naman.a@nus.edu.sg>
Repository: CRAN
Date/Publication: 2026-04-11 12:00:12 UTC
Built: R 4.6.0; ; 2026-04-28 23:27:19 UTC; unix
