# Generated by roxygen2: do not edit by hand

export("%>%")
export(K2)
export(TreeHarp)
export(apply_can_improve_this)
export(carve_mst)
export(carve_subtree)
export(check_correctness)
export(check_rmd)
export(check_runtime)
export(clean_dir)
export(copy_e2e)
export(count_fn_call)
export(count_fn_in_fn)
export(count_lam_fn)
export(count_lints_all)
export(count_lints_one)
export(count_num_lines_for_loop)
export(count_self_fn)
export(detect_fn_arg)
export(detect_fn_call_in_for)
export(detect_for_in_fn_def)
export(detect_growing)
export(detect_nested_for)
export(env_size)
export(extract_actual_args)
export(extract_assigned_objects)
export(extract_chunks)
export(extract_fn_call)
export(extract_formal_args)
export(extract_non_chunks)
export(extract_section_text)
export(extract_self_fn)
export(fapply)
export(find_branch_num)
export(generate_all_subtrees)
export(generate_thumbnails)
export(get_adj_list)
export(get_child_ids)
export(get_libraries)
export(get_next_subtree)
export(get_node_types)
export(get_parent_call_id)
export(get_parent_id)
export(get_recursive_index)
export(is_subtree_rooted_at)
export(jaccard_treeharp)
export(join_treeharps)
export(keep_branches)
export(lang_2_tree)
export(log_summary)
export(only_actual_args_differ)
export(path_to_root)
export(pipe_can_improve_this)
export(populate_soln_env)
export(prune_depth)
export(render_one)
export(render_prechecks)
export(replace_sp_chars_filename)
export(reset_path)
export(rmd_to_forestharp)
export(rmd_to_token_count)
export(subtree_at)
exportMethods(TreeHarp)
exportMethods(get_adj_list)
exportMethods(get_child_ids)
exportMethods(get_node_types)
exportMethods(get_parent_id)
exportMethods(length)
exportMethods(names)
exportMethods(plot)
exportMethods(show)
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setValidity)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(parallel,clusterCall)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,makePSOCKcluster)
importFrom(parallel,stopCluster)
importFrom(rlang,.data)
importFrom(rlang,call_args)
importFrom(rlang,call_args_names)
importFrom(rlang,call_name)
importFrom(rlang,enexpr)
importFrom(rlang,is_formula)
importFrom(rlang,is_na)
importFrom(rlang,is_pairlist)
importFrom(rlang,is_symbol)
importFrom(rlang,is_syntactic_literal)
importFrom(rmarkdown,render)
importFrom(stats,na.omit)
importFrom(utils,tail)
