#' autoharp: Semi-Automatic Grading of R and Rmd Scripts
#'
#' The autoharp package provides functions for running and analysing 
#' R script/Rmd submissions from students.
#' 
#' @section Main functions are:
#' \enumerate{
#'   \item \code{\link{populate_soln_env}}
#'   \item \code{\link{render_one}}
#'   \item \code{\link{TreeHarp}}
#' }
#' 
#' The user manuals can be found at \url{https://namanlab.github.io/autoharp/}
#'
#' @name autoharp
"_PACKAGE"
