unlink("test01.Rmd")
unlink("test02.Rmd")
unlink("test03.Rmd")
unlink("test03.html")
unlink("test04.Rmd")
unlink("soln_template.Rmd")
unlink("s1.R")
unlink("s2.R")
unlink("s3.R")
unlink("s4.R")
unlink("render_all.log")
unlink("test_output", TRUE, TRUE)
