---
title: "Solution Template (02)"
output: html_document
---
# Summary Output

This is a summary of the tests conducted in this solution template.

1.  `run_status`: SUCCESS indicates that I am able to run your code - 
    there are no syntax errors, and path settings are correct.
2.  `last.X`: This entry displays the last value of the `X` vector in your code.
    An NA indicates that the vector was not of the correct length, or that the 
    vector is not present.

```{r test00}
library(testthat)
library(autoharp)
```

```{r corr_solutions, echo=FALSE}
X <- c(1L, 3L, 5L, 7L,  9L, 11L)
mean_X <- mean(X)
```

```{r test02, autoharp.scalars=c("last.X")}
last.X <- X[length(X)]
```



